package com.sri;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sonia.scm.client.RepositoryClientHandler;
import sonia.scm.client.ScmClient;
import sonia.scm.client.ScmClientSession;
import sonia.scm.repository.Repository;

import javax.net.ssl.*;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Map;


/**
 * Hello world!
 *
 */
public class App implements Runnable
{

    @Option(name="-u", usage = "SCM Manager username", required = true)
    private String username;

    @Option(name="-p", usage = "SCM Manager password", required = true)
    private String password;

    @Option(name="-s", usage = "SCM Manager URL", hidden = true)
    private String url = "https://scm.sri.com/scm";

    @Option(name="-d", usage = "Disable SSL certificate check check")
    private boolean disableSSL;

    private static final Logger logger = LoggerFactory.getLogger(App.class);


    public App(String[] args) throws Exception {
        CmdLineParser parser = new CmdLineParser(this);
        try {
            parser.parseArgument(args);

            if (disableSSL) {
                disableSslVerification();
            }
        } catch (CmdLineException cle) {
            System.err.println(cle.getMessage());
            System.err.println(String.format("java %s [options]", App.class.getSimpleName()));
            parser.printUsage(System.err);
            throw new Exception(cle);
        }
    }

    private void disableSslVerification() {
        try
        {
            // Create a trust manager that does not validate certificate chains
            TrustManager[] trustAllCerts = new TrustManager[] {new X509TrustManager() {
                public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
                public void checkClientTrusted(X509Certificate[] certs, String authType) {
                }
                public void checkServerTrusted(X509Certificate[] certs, String authType) {
                }
            }
            };

            // Install the all-trusting trust manager
            SSLContext sc = SSLContext.getInstance("SSL");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());

            // Create and Install the all-trusting host verifier
            HttpsURLConnection.setDefaultHostnameVerifier((s, sslSession) -> {
                return true;
            });
        } catch (NoSuchAlgorithmException e) {
            logger.error("Problem with disabling SSL.", e);
        } catch (KeyManagementException e) {
            logger.error("Problem with key management while disabling SSL.", e);
        }
    }


    @Override
    public void run() {
        ScmClientSession session = ScmClient.createSession(this.url, this.username, this.password);
        RepositoryClientHandler repositoryHandler = session.getRepositoryHandler();

        List<Repository> repositoryList = repositoryHandler.getAll();

        repositoryList.forEach(r -> {
            Map<String, String> props = r.getProperties();
            if (props.size() == 0) {
                System.err.println(String.format("%s has no properties.", r.createUrl(this.url)));
            } else {
                System.out.println(String.format("Properties for %s", r.createUrl(this.url)));
                props.forEach((s, s2) -> System.out.println(String.format("key: %s, value: %s", s, s2)));
            }
        });
    }

    public static void main( String[] args )
    {
        try {
            App app = new App(args);
            app.run();
        } catch (Exception e) {
            logger.error("Problem running demo.", e);
        }
    }
}
